<?php

namespace Teknisa\TCO\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DataSource;

class Tco01003DadosPrato extends \Teknisa\GER\Controllers\TeknisaCrud {
    
    protected $crudService;
    protected $environment;
    protected $entityManager;

    public function __construct($crudService, $parameterBag, $environment, $entityManager) {
        $this->crudService = $crudService;
        $this->environment = $environment;
        $this->entityManager = $entityManager;
    }
    
    public function removePratCompCardBase(DTO\Request $request, DTO\Response $response) {
        $row = $request->getRow();
        $this->verificaInativaRegraDia($row, $response);
        $this->removeRegiaoPratCardBase($this->retornaRegiaoPratCardBase($row), $response);
        $this->removePratCompCardBaseSelecionado($request, $response);
    }
    
    private function verificaInativaRegraDia($row, $response) {
        $regras = $this->retornaRegraDiaEmUso($row);
        if(sizeof($regras)) {
            $this->inativaRegra($regras);
            $response->addDataSet(new DataSet("adesaoCardBaseInativada", $regras));
            $response->addMessage(new Message('TCO01000_ADESAOCARDBASE_INATIVADA'));
        }
    }
    
    private function retornaRegraDiaEmUso($row) {
        $connection = $this->entityManager->getConnection();
        
        $sql = "SELECT NRADESAOCARDBASE,
                       NRCARDAPIOBASE,
                       CDADESAOCARDBASE,
                       NMADESAOCARDBASE,
                       CDGRTIPOPR,
                       NRDIACARDBASE,
                       NRORG,
                       IDATIVO
                  FROM ADESAOCARDBASE
                 WHERE NRCARDAPIOBASE = :NRCARDAPIOBASE 
                   AND NRDIACARDBASE = :NRDIACARDBASE
                   AND IDATIVO = 'S'
                   AND NRORG = :NRORG";
        
        $query = $connection->prepare($sql);
        
        $params = array(
            "NRCARDAPIOBASE"  => $row["NRCARDAPIOBASE"],
            "NRDIACARDBASE"  => $row["NRDIACARDBASE"],
            "NRORG"  => $this->environment->getNrOrgTrab()
        ); 
        
        $query->execute($params);
        
        $result = $query->fetchAll(\PDO::FETCH_ASSOC);
        
        return $result;
    }
    
    private function inativaRegra($regras) {
        $regras[0]["IDATIVO"] = "N";
        $regras[0]["__is_new"] = false;
        $regras[0]["changed"] = true;
        
        $dataSet = new DataSet("adesaocardbase", $regras);
        $request = new DTO\Request\DataSet($dataSet, "", "", "");
        $rows = $this->crudService->getRowsToSave($request, "adesaocardbase");
        $this->crudService->save("adesaocardbase", $rows);
    }
    
    private function retornaRegiaoPratCardBase ($row) {
        $connection = $this->entityManager->getConnection();
        
        $sql = "SELECT NRREGIAOPRATCARDBASE,
                       NRPRATCOMPCARDBASE,
                       CDREGIAO,
                       NRORG
                  FROM REGIAOPRATCARDBASE
                 WHERE NRPRATCOMPCARDBASE = :NRPRATCOMPCARDBASE
                   AND NRORG = :NRORG";
        
        $query = $connection->prepare($sql);
        
        $params = array(
            "NRPRATCOMPCARDBASE"  => $row["NRPRATCOMPCARDBASE"],
            "NRORG"  => $this->environment->getNrOrgTrab()
        ); 
        
        $query->execute($params);
        
        $result = $query->fetchAll(\PDO::FETCH_ASSOC);
        
        return $result;
    }
    
    private function removeRegiaoPratCardBase($rows, $response) {
        $dataSet = new DataSet("regiaopratcardbase", $rows);
        $request = new DTO\Request\DataSet($dataSet, "", "", "");
        $rows = $this->crudService->getRowsToDelete($request, "regiaopratcardbase");
        $dataSetToDelete = new DataSet("regiaopratcardbase", $rows);
		$this->crudService->delete($response, $dataSetToDelete);
    } 
    
    private function removePratCompCardBaseSelecionado($request, $response) {
        $row = $request->getRow();
        $rows = $this->crudService->getRowsToDelete($request, "pratcompcardbase");
        $dataSet = new DataSet("pratcompcardbase", $rows);
		$this->crudService->delete($response, $dataSet);
        if($row["IDPRATOEXIBICAO"] === "S"){
            $this->definePrimeiraOcorrenciaReferencia($row);
        }
    }
    
    public function definirReferenciaPrato(DTO\Request $request, DTO\Response $response) {
        $row = $request->getRow();
        $rowsToSave = array();
        $row["IDPRATOEXIBICAO"] = "S";
        $row["__is_new"] = false;
        $row["changed"] = true;
        array_push($rowsToSave, $row);
        
        $referenciaAntiga = $this->pratoExibicaoOcorrencia($row);
        if(sizeof($referenciaAntiga)) {
            $referenciaAntiga[0]["IDPRATOEXIBICAO"] = "N";
            $referenciaAntiga[0]["__is_new"] = false;
            $referenciaAntiga[0]["changed"] = true;
            array_push($rowsToSave, $referenciaAntiga[0]);
        }
        
        $dataSet = new DataSet("pratcompcardbase", $rowsToSave);
        $novoRequest = new DTO\Request\DataSet($dataSet, "", "", "");
        $rows = $this->crudService->getRowsToSave($novoRequest, "pratcompcardbase");
        $this->crudService->save("pratcompcardbase", $rows);
        $response->addDataSet(new DataSet("pratcompcardbase", $rows));
    }
    
    private function pratoExibicaoOcorrencia($row) {
        $connection = $this->entityManager->getConnection();
        
        $sql = "SELECT PRATCOMPCARDBASE.NRPRATCOMPCARDBASE,
                       PRATCOMPCARDBASE.CDPRATO,
                       PRATCOMPCARDBASE.CDFILAUXPRAT,
                       PRATCOMPCARDBASE.IDPRATOEXIBICAO,
                       PRATCOMPCARDBASE.NRORG
                  FROM PRATCOMPCARDBASE
                 INNER JOIN COMPCARDBASE
                    ON COMPCARDBASE.NRCOMPCARDBASE         = PRATCOMPCARDBASE.NRCOMPCARDBASE
                 WHERE COMPCARDBASE.NRCARDAPIOBASE         = :NRCARDAPIOBASE
                   AND COMPCARDBASE.NRDIACARDBASE          = :NRDIACARDBASE
                   AND COMPCARDBASE.NRCOMPCARDBASE         = :NRCOMPCARDBASE
                   AND COMPCARDBASE.NRCONFIGCARDBASE       = :NRCONFIGCARDBASE
                   AND PRATCOMPCARDBASE.NRPRATCOMPCARDBASE <> :NRPRATCOMPCARDBASE
                   AND PRATCOMPCARDBASE.NRORG              = :NRORG
                   AND PRATCOMPCARDBASE.IDPRATOEXIBICAO    = 'S'";
        
        $query = $connection->prepare($sql);
        
        $params = array(
            "NRCARDAPIOBASE"     => $row["NRCARDAPIOBASE"],
            "NRDIACARDBASE"      => $row["NRDIACARDBASE"],
            "NRCOMPCARDBASE"     => $row["NRCOMPCARDBASE"],
            "NRCONFIGCARDBASE"   => $row["NRCONFIGCARDBASE"],
            "NRPRATCOMPCARDBASE" => $row["NRPRATCOMPCARDBASE"],
            "NRORG"              => $this->environment->getNrOrgTrab()
        ); 
        
        $query->execute($params);
        
        $result = $query->fetchAll(\PDO::FETCH_ASSOC);
        
        return $result;
    }
    
    private function definePrimeiraOcorrenciaReferencia($row) {
        $novaReferencia = array();
        $pratoParaReferencia = $this->retornaPratoDisponivel($row);
        
        if(!empty($pratoParaReferencia)) {
            
            $pratoParaReferencia["IDPRATOEXIBICAO"] = "S";
            $pratoParaReferencia["changed"] = true;
            $pratoParaReferencia["__is_new"] = false;
            
            array_push($novaReferencia, $pratoParaReferencia);
            
            $dataSet = new DataSet("pratcompcardbase", $novaReferencia);
            $request = new DTO\Request\DataSet($dataSet, "", "", "");
            $rows = $this->crudService->getRowsToSave($request, "pratcompcardbase");
            $this->crudService->save("pratcompcardbase", $rows);
            
        }
    }
    
    private function retornaPratoDisponivel($row) {
        $connection = $this->entityManager->getConnection();
        
        $sql = "SELECT NRPRATCOMPCARDBASE,
                       NRCOMPCARDBASE,
                       CDFILAUXPRAT,
                       CDPRATO,
                       IDPRATOEXIBICAO,
                       NRORG
                  FROM PRATCOMPCARDBASE
                 WHERE NRCOMPCARDBASE = :NRCOMPCARDBASE
                   AND NRORG = :NRORG
                 ORDER BY CDPRATO ASC";
        
        $query = $connection->prepare($sql);
        
        $params = array(
            "NRCOMPCARDBASE"  => $row["NRCOMPCARDBASE"],
            "NRORG"  => $this->environment->getNrOrgTrab()
        ); 
        
        $query->execute($params);
        
        $result = $query->fetch(\PDO::FETCH_ASSOC);
        
        return $result;
    }

}
